using Nostr.Client.Messages;
using Nostr.Client.Requests;
using NostrRelay;

namespace NostrServices.Services;

public class CacheRelay : INostrRelay
{
    private readonly ILogger<CacheRelay> _logger;
    private readonly RedisStore _redisStore;

    public CacheRelay(RedisStore redisStore, ILogger<CacheRelay> logger)
    {
        _redisStore = redisStore;
        _logger = logger;
    }

    public ValueTask<bool> AcceptConnection(NostrClientContext context)
    {
        _logger.LogInformation("New connection {ip} {ua}", context.Ip, context.UserAgent);
        return ValueTask.FromResult(true);
    }

    public async IAsyncEnumerable<NostrEvent> HandleRequest(NostrClientContext context, NostrRequest req)
    {
        //no results yet
        yield break;
    }

    public async ValueTask<HandleEventResponse> HandleEvent(NostrClientContext context, NostrEvent ev)
    {
        if (RelayListener.AcceptedKinds.Contains(ev.Kind))
        {
            await _redisStore.StoreEvent(CompactEvent.FromNostrEvent(ev));
            return new(true, null);
        }

        return new(false, "blocked: kind not accepted");
    }
}
